@@ -1,7 +1,7 @@
 use super::{
-    transaction_chain, CancelTransaction, DepositTransaction, EmergencyAddress,
-    EmergencyTransaction, FeeBumpTransaction, RevaultTransaction, SpendTransaction,
-    UnvaultEmergencyTransaction, UnvaultTransaction, DUST_LIMIT,
+    transaction_chain, CancelTransaction, CpfpTransaction, CpfpableTransaction, DepositTransaction,
+    EmergencyAddress, EmergencyTransaction, FeeBumpTransaction, RevaultTransaction,
+    SpendTransaction, UnvaultEmergencyTransaction, UnvaultTransaction, CPFP_MIN_CHANGE, DUST_LIMIT,
 };
 
 use crate::{error::*, scripts::*, txins::*, txouts::*};
@@ -11,6 +11,7 @@ use std::{iter::repeat_with, str::FromStr};
 use miniscript::{
     bitcoin::{
         secp256k1,
+        util::psbt::PartiallySignedTransaction as Psbt,
         util::{bip143::SigHashCache, bip32},
         Address, Amount, Network, OutPoint, SigHash, SigHashType, Transaction, TxIn, TxOut,
     },
@@ -143,6 +144,54 @@ fn satisfy_transaction_input(
     Ok(())
 }
 
+fn sign_psbt(
+    secp: &secp256k1::Secp256k1<secp256k1::All>,
+    psbt: &mut Psbt,
+    input_index: usize,
+    xprivs: &[bip32::ExtendedPrivKey],
+    child_number: bip32::ChildNumber,
+) {
+    for xpriv in xprivs {
+        let deriv_path = bip32::DerivationPath::from(vec![child_number]);
+        let sig = {
+            let prev_amount = psbt.inputs[input_index]
+                .witness_utxo
+                .as_ref()
+                .unwrap()
+                .value;
+            let script_code = psbt.inputs[input_index].witness_script.as_ref().unwrap();
+            let sighash = SigHashCache::new(&psbt.global.unsigned_tx).signature_hash(
+                input_index,
+                &script_code,
+                prev_amount,
+                SigHashType::All,
+            );
+            secp.sign(
+                &secp256k1::Message::from_slice(&sighash).unwrap(),
+                &xpriv
+                    .derive_priv(&secp, &deriv_path)
+                    .unwrap()
+                    .private_key
+                    .key,
+            )
+        };
+
+        let xpub = bip32::ExtendedPubKey::from_private(&secp, xpriv);
+        let pubkey = xpub.derive_pub(secp, &deriv_path).unwrap();
+        let mut sig = sig.serialize_der().to_vec();
+        sig.push(SigHashType::All.as_u32() as u8);
+        psbt.inputs[input_index]
+            .partial_sigs
+            .insert(pubkey.public_key, sig);
+    }
+}
+
+fn finalize_psbt(secp: &secp256k1::Secp256k1<impl secp256k1::Verification>, psbt: &mut Psbt) {
+    miniscript::psbt::finalize(psbt, secp)
+        .map_err(|e| Error::TransactionFinalisation(e.to_string()))
+        .unwrap();
+}
+
 fn desc_san_check<P: MiniscriptKey>(desc: &Descriptor<P>) -> Result<(), ScriptCreationError> {
     match desc {
         Descriptor::Wsh(wsh) => wsh.sanity_check().map_err(|e| e.into()),
@@ -185,7 +234,7 @@ pub fn derive_transactions(
     // Keys, keys, keys everywhere !
     let (
         (managers_priv, managers),
-        (_, mancpfp),
+        (mancpfp_priv, mancpfp),
         (stakeholders_priv, stakeholders),
         (cosigners_priv, cosigners),
     ) = get_participants_sets(n_stk, n_man, with_cosig_servers, secp);
@@ -617,6 +666,73 @@ pub fn derive_transactions(
     unvault_tx.finalize(&secp)?;
     roundtrip!(unvault_tx, UnvaultTransaction);
 
+    // Create a CPFP transaction for the unvault
+    // Some fake listunspent outputs
+    let listunspent = vec![
+        CpfpTxIn::new(
+            OutPoint::from_str(
+                "f21596dd9df36b86bcf65f0884f1f20675c1fc185bc78a37a9cddb4ae5e3dd9f:0",
+            )
+            .unwrap(),
+            CpfpTxOut::new(Amount::from_sat(30_000), &der_cpfp_descriptor),
+        ),
+        CpfpTxIn::new(
+            OutPoint::from_str(
+                "f21596dd9df36b86bcf65f0884f1f20675c1fc185bc78a37a9cddb4ae5e3dd9f:1",
+            )
+            .unwrap(),
+            CpfpTxOut::new(Amount::from_sat(30_000), &der_cpfp_descriptor),
+        ),
+    ];
+
+    let cpfp_txins = vec![
+        unvault_tx.cpfp_txin(&der_cpfp_descriptor).unwrap(),
+        unvault_tx.cpfp_txin(&der_cpfp_descriptor).unwrap(),
+    ];
+    let tbc_weight = unvault_tx.max_weight() * 2;
+    let tbc_fees = Amount::from_sat(unvault_tx.fees() * 2);
+    // Let's ask for a decent feerate
+    let added_feerate = 6121;
+    // We try to feebump two unvaults in 1 transaction
+    let cpfp_tx = CpfpTransaction::from_txins(
+        cpfp_txins.clone(),
+        tbc_weight,
+        tbc_fees,
+        added_feerate,
+        listunspent.clone(),
+    )
+    .unwrap();
+
+    // The cpfp tx contains the input of the tx to be cpfped, right?
+    assert!(cpfp_tx.tx().input.contains(&cpfp_txins[0].unsigned_txin()));
+
+    assert_eq!(cpfp_tx.tx().output.len(), 1);
+    {
+        let o = &cpfp_tx.tx().output[0];
+        // Either the change is 0 with an OP_RETURN,
+        // or its value is bigger than CPFP_MIN_CHANGE, and we send
+        // back to the cpfp_txin script_pubkey
+        assert!(
+            (o.value == 0 && o.script_pubkey.is_op_return())
+                || (o.value >= CPFP_MIN_CHANGE
+                    && o.script_pubkey == cpfp_txins[0].txout().txout().script_pubkey)
+        );
+    }
+
+    // we sign the cpfp and then check the package feerate
+    let cpfp_fees = cpfp_tx.fees();
+    let inputs_len = cpfp_tx.psbt().inputs.len();
+    let mut psbt = cpfp_tx.into_psbt();
+    for i in 0..inputs_len {
+        sign_psbt(&secp, &mut psbt, i, &mancpfp_priv, child_number);
+    }
+    finalize_psbt(&secp, &mut psbt);
+    assert!(
+        1000 * (cpfp_fees + unvault_tx.fees())
+            / (psbt.global.unsigned_tx.get_weight() as u64 + unvault_tx.max_weight())
+            >= unvault_tx.max_feerate() * 1000 + added_feerate
+    );
+
     // Create and sign a spend transaction
     let spend_unvault_txin = unvault_tx.spend_unvault_txin(&der_unvault_descriptor);
     let unvault_value = spend_unvault_txin.txout().txout().value;
@@ -794,6 +910,69 @@ pub fn derive_transactions(
             Some(child_number),
         )?
     }
+
+    // Create a CPFP transaction for the (not yet finalized) Spend
+    // Some fake listunspent outputs
+    let listunspent = vec![
+        CpfpTxIn::new(
+            OutPoint::from_str(
+                "f21596dd9df36b86bcf65f0884f1f20675c1fc185bc78a37a9cddb4ae5e3dd9f:0",
+            )
+            .unwrap(),
+            CpfpTxOut::new(Amount::from_sat(58_000), &der_cpfp_descriptor),
+        ),
+        CpfpTxIn::new(
+            OutPoint::from_str(
+                "f21596dd9df36b86bcf65f0884f1f20675c1fc185bc78a37a9cddb4ae5e3dd9f:1",
+            )
+            .unwrap(),
+            CpfpTxOut::new(Amount::from_sat(23_000), &der_cpfp_descriptor),
+        ),
+    ];
+
+    let cpfp_txin = spend_tx.cpfp_txin(&der_cpfp_descriptor).unwrap();
+    let cpfp_txins = vec![cpfp_txin.clone()];
+    let tbc_weight = spend_tx.max_weight();
+    let tbc_fees = Amount::from_sat(spend_tx.fees());
+    let added_feerate = 6121;
+    let cpfp_tx = CpfpTransaction::from_txins(
+        cpfp_txins,
+        tbc_weight,
+        tbc_fees,
+        added_feerate,
+        listunspent.clone(),
+    )
+    .unwrap();
+
+    // The cpfp tx contains the input of the tx to be cpfped
+    assert!(cpfp_tx.tx().input.contains(&cpfp_txin.unsigned_txin()));
+    assert_eq!(cpfp_tx.tx().output.len(), 1);
+    // Either the change is 0 with an OP_RETURN,
+    // or its value is bigger than CPFP_MIN_CHANGE, and we send
+    // back to the cpfp_txin script_pubkey
+    {
+        let o = &cpfp_tx.tx().output[0];
+        assert!(
+            (o.value == 0 && o.script_pubkey.is_op_return())
+                || (o.value >= CPFP_MIN_CHANGE
+                    && o.script_pubkey == cpfp_txin.txout().txout().script_pubkey)
+        );
+    }
+
+    // we sign the cpfp and then check the package feerate
+    let cpfp_fees = cpfp_tx.fees();
+    let inputs_len = cpfp_tx.psbt().inputs.len();
+    let mut psbt = cpfp_tx.into_psbt();
+    for i in 0..inputs_len {
+        sign_psbt(&secp, &mut psbt, i, &mancpfp_priv, child_number);
+    }
+    finalize_psbt(&secp, &mut psbt);
+    assert!(
+        1000 * (cpfp_fees + spend_tx.fees())
+            / (psbt.global.unsigned_tx.get_weight() as u64 + spend_tx.max_weight())
+            >= spend_tx.max_feerate() * 1000 + added_feerate
+    );
+
     roundtrip!(spend_tx, SpendTransaction);
     spend_tx.finalize(&secp)?;
     roundtrip!(spend_tx, SpendTransaction);
